/* lsystem4 -- simple lindenmayer system with audio realization of the
	graph using RTcmix WAVETABLE instruments (direct frequency
	interpretation of pixel values of red lines).  Uses "MYWAVETABLE"
	instrument to turn on & off instruments and dynamically set
	frequencies.  This is done by simply setting an amp var in
	MYWAVETABLE to 1.0 or 0.0 on a running instrument.  "myTimer.C
	is the animator again, and sets the instruments in motion
	-- BGG/2003
*/
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
	#include "wx/wx.h"
#endif

#define MAIN		/* So RTcmix globals will be right */
#include <globals.h>

#include "lsystem4.h"
#include <iostream.h>


IMPLEMENT_APP(lsystem4)

bool lsystem4::OnInit()
{
	wxFrame *frame = new wxFrame((wxFrame*) NULL, -1, "lsystem4",
		wxPoint(50, 50), wxSize(1000, 600),
		wxDEFAULT_FRAME_STYLE, "lsystem4");
	frame->Show(TRUE);
	SetTopWindow(frame);

	wxButton *button1 = new wxButton(frame, BUTTON_HIT1, "realize!",
		wxPoint(100, 550), wxSize(70, 20));

	wxButton *button2 = new wxButton(frame, BUTTON_HIT2, "go!",
		wxPoint(200, 550), wxSize(70, 20));

	wdc = new wxClientDC(frame);
	recurse_levels = 5;

	RTcmix *rrr = new RTcmix(44100.0, 2);
	sleep(1);
	rrr->cmd("load", 1, "./MYWAVETABLE/libMYWAVETABLE.so");
	rrr->cmd("makegen", 7, 1.0, 24.0, 1000.0, 0.0, 1.0, 1.0, 1.0);
	rrr->cmd("makegen", 5, 2.0, 10.0, 1000.0, 1.0);
	rrr->cmd("reset", 1, 44100.0);

	theTimer = new myTimer(wdc, rrr);

	return true;
}

void lsystem4::OnButton1(wxCommandEvent& command)
{
	int x, y, length;

	x = 0;
	y = 250;
	length = 1000/6;

	wdc->SetPen(*wxRED_PEN);
	wdc->DrawLine(x,y, length,y);
	wdc->DrawLine(x,y, length,y-70);

	recurse(x+length, y, length/2, recurse_levels-1);
	recurse(x+length, y-70, length/3, recurse_levels-1);
}


void lsystem4::OnButton2(wxCommandEvent& command)
{
	// 100 msec timing
	theTimer->setX(0);
	theTimer->Start(100, wxTIMER_CONTINUOUS);
}

void lsystem4::recurse(int x, int y, int length, int rlevel)
{
	if (rlevel == 0) return;

	wdc->SetPen(*wxRED_PEN);
	wdc->DrawLine(x,y, x+length,y+20);
	wdc->DrawLine(x,y, x+length,y-50);
	wdc->DrawLine(x,y, x+length-10,y-10);

	recurse(x+length, y+20, length-rlevel, rlevel-1);
	recurse(x+length, y-50, length/2, rlevel-1);
	recurse(x+length-10, y-10, length/2, rlevel-1);
}

BEGIN_EVENT_TABLE(lsystem4, wxApp)
	EVT_BUTTON(BUTTON_HIT1, lsystem4::OnButton1)
	EVT_BUTTON(BUTTON_HIT2, lsystem4::OnButton2)
END_EVENT_TABLE()
